function [modifiedChain, errorTerm] = modifyDihedral(chain, target)
% [modifiedChain, errorTerm] = modifyDihedral(chain, target)
% returns the modified chain, while minimizing the least squares distance
% of the last atoms of the chain to the target. The target can be either
% one atom, or three atoms. In the latter case there is also a orientation
% term being considered in the objective function.
%
% If target consists of three atoms, then the bond lengths have to be the
% same as between the last three atoms of the chain, i.e. target must be
% congruent to the last three atoms of the chain.
%
% The chain is modified in the following way: The first four atoms of the
% chain define a dihedral angle. This angle is modified freely, so the rest
% of the chain is rotated around the axis defined by the second and third
% atom in the chain. The rotation angle is chosed to minimize the error term
%
%	S = d^2
%
% in the case of one atom being matched, where d is the distance of the
% last atom in the chain and the target atom.
% In the case of three atom being matched the error term looks as follows,
%
%	S = d1^2 + d2^2 + d3^2 + 2 + v1 + v2
%
% where d1 to d3 are the distances of the chain atoms to their targets, and
% v1 and v2 are dot products of two unit vectors. Therefore v1+v2 is at least
% -2 and the error term is positive.

n = size(chain,1);
nTarget = size(target, 1);
match3 = false; % match only one atom per default
if (nTarget == 3)
	match3 = true;
else
	if (nTarget ~= 1)
		n = -1; % prevent the following code from being executed (target ill-specified)
	end
end


if (n > 3) 
    
    % compute Theta vector of local coord systems
    lSysT = chain(3,:) - chain(2,:);
    lSysT = lSysT / norm(lSysT);
    
    % compute origins of local coord systems
    lOrig = zeros(nTarget,3);
    lambda = dot(lSysT, chain(n-2,:) - chain(3,:));
    lOrig(1,:) = chain(3,:) + lambda * lSysT;
    if (match3)
    	lambda = dot(lSysT, chain(n-1,:) - chain(3,:));
    	lOrig(2,:) = chain(3,:) + lambda * lSysT;
    	lambda = dot(lSysT, chain(n,:) - chain(3,:));
    	lOrig(3,:) = chain(3,:) + lambda * lSysT;
	end
	
    % compute other vectors of local coordinate systems
    lChain = chain(n-nTarget+1:n,:) - lOrig;
    chainLenSq = sum(lChain.^2, 2);
    lSysR = lChain ./ repmat(sqrt(chainLenSq),1,3);
    % check if point lies on rotation axis (dangerous because origin=point)
    index = (chainLenSq == 0);
    if any(index)
        loop = find(index);
        m = size(loop,1);
        firstCoord = - (lSysT(2)+lSysT(3)) / lSysT(1);
        repl = [firstCoord, 1, 1];
        repl = repl / norm(repl);
        lSysR(index,:) = repmat(repl,sum(index),1);
    end
    lSysS = cross(lSysR, repmat(lSysT, nTarget, 1), 2);
    
    % compute coefficients of objective function S = a - b cos(Theta) - c sin(Theta)
    lTarget = target - lOrig;
    targetLenSq = sum(lTarget.^2, 2);
  
    
    a = sum(chainLenSq) + sum(targetLenSq);
    b = 2 * dot(sqrt(chainLenSq)', diag(lTarget * lSysR'));
    c = 2 * dot(sqrt(chainLenSq)', diag(lTarget * lSysS'));
    
    
    % add orientation term to energy function if three atoms are matched
    if (match3)
        tmpV1 = lTarget(1,:) - lTarget(2,:);
        tmpV2 = lTarget(3,:) - lTarget(2,:);
        tmpR = sqrt(chainLenSq(2)) * lSysR(2,:);
        tmpS = sqrt(chainLenSq(2)) * lSysS(2,:);
        bContrib = dot( sqrt(chainLenSq(1)) * lSysR(1,:) - tmpR, tmpV1) / norm(tmpV1)^2 ...
               + dot( sqrt(chainLenSq(3)) * lSysR(3,:) - tmpR, tmpV2) / norm(tmpV2)^2;
        cContrib = dot( sqrt(chainLenSq(1)) * lSysS(1,:) - tmpS, tmpV1) / norm(tmpV1)^2 ...
               + dot( sqrt(chainLenSq(3)) * lSysS(3,:) - tmpS, tmpV2) / norm(tmpV2)^2;
        a = a + 2;
        b = b + bContrib;
        c = c + cContrib;
    end

    
    % compute Theta
    cosAlpha = b / (sqrt(b^2 + c^2));
    sinAlpha = c / (sqrt(b^2 + c^2));
    Theta = atan2(sinAlpha, cosAlpha);


    
    % compute global coordinates of rotated points and return result
    modifiedChain = rotateDihedral(Theta, chain);
    errorTerm = abs(sqrt(a - b * cos(Theta) - c * sin(Theta)));

else % input was ill-specified, return original chain and error=0
	modifiedChain = chain;
	errorTerm = 0;
end



